package com.plannaplan.responses.models;

import com.plannaplan.entities.User;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**
 * Response to show after successfully login cation
 */
@ApiModel(description = "Response shows information about logged user.", value = "TokenResponse")
public class TokenResponse {
    @ApiModelProperty(value = "user token used to verify requests")
    private String token;
    @ApiModelProperty(value = "user token needed to refresh")
    private String refreshToken;
    @ApiModelProperty(value = "user id in database")
    private Long id;
    @ApiModelProperty(value = "user app role")
    private String authorityRole;
    @ApiModelProperty(value = "user unviersity email")
    private String email;

    /**
     * @param user user to be mapped to api response
     */
    public TokenResponse(User user) {
        this.id = user.getId();
        this.authorityRole = user.getRole().toString();
        this.email = user.getEmail();
        this.token = user.getToken();
        this.refreshToken = user.getRefreshToken();
    }

    /**
     * @return user email
     */
    public String getEmail() {
        return email;
    }

    /**
     * @return user role in system
     */
    public String getAuthorityRole() {
        return authorityRole;
    }

    /**
     * @return db id
     */
    public Long getId() {
        return id;
    }

    /**
     * @return user token to authorize other requests
     */
    public String getToken() {
        return token;
    }

    /**
     * @return user refresh token
     */
    public String getRefreshToken() {
        return this.refreshToken;
    }

}
